package com.ifgi.gnc.client;


import java.util.ArrayList;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.Window.Location;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.ifgi.gnc.server.TaskHandler;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class GNC implements EntryPoint {
	
	private ProductInfoServiceAsync productInfoSvc = GWT.create(ProductInfoService.class);
	private ProductsListServiceAsync productsListSvc = GWT.create(ProductsListService.class);
	private ActivateTaskServiceAsync activateTaskSvc = GWT.create(ActivateTaskService.class);

	final Grid productListGrid = new Grid(1, 2);
	final DialogBox refreshProductListDialogBox = new DialogBox();
	final DialogBox productUpdateDialogBox = new DialogBox();
	final DialogBox activateTaskDialogBox = new DialogBox();
	final DockPanel mainLayout = new DockPanel();
	final VerticalPanel vPanel = new VerticalPanel();
	final ListBox productChooser = new ListBox();
		
	/**
	 * This is the entry point method.
	 */
	public void onModuleLoad() {
		mainLayout.setStylePrimaryName("mainpan");
		
		mainLayout.add(new Credits(), DockPanel.WEST);
		
		if (Location.getParameter("activate") != null) {
			activeTask(Location.getParameter("activate"));
		} else if (Location.getParameter("folder") != null){
			mainLayout.add(new UserFolder(Location.getParameter("folder")), DockPanel.CENTER);
			RootPanel.get("Scheduler").add(mainLayout);
		} else {
			mainPanel();
		}
	}
	
	void mainPanel (){
		//Refresh Products Dialog Box
		refreshProductListDialogBox.setText("Refreshing Product List");
		refreshProductListDialogBox.setAnimationEnabled(true);
		Label productListRefreshLabel = new Label("Please wait while the product list refreshes");
		productListRefreshLabel.setStylePrimaryName("dbtext");
		refreshProductListDialogBox.add(productListRefreshLabel);
		refreshProductListDialogBox.center();
		
		//Refresh Product Info
		productUpdateDialogBox.setText("Refreshing Product Information");
		productUpdateDialogBox.setAnimationEnabled(true);
		Label productRefreshLabel = new Label("Please wait while the product information refreshes");
		productRefreshLabel.setStylePrimaryName("dbtext");
		productUpdateDialogBox.add(productRefreshLabel);
		productUpdateDialogBox.center();
				
		Label availProLabel = new Label("Available Products:");
		availProLabel.setStylePrimaryName("listlabel");
		productListGrid.setWidget(0, 0, availProLabel);
				
		requestProductsList();
		vPanel.setStylePrimaryName("vertpan");
		vPanel.add(productListGrid);
		
		productChooser.setStylePrimaryName("procho");
		
		
		mainLayout.add(vPanel, DockPanel.CENTER);
		RootPanel.get("Scheduler").add(mainLayout);
	}

	private void requestProductsList() {
	    if (productsListSvc == null) {
	    	productsListSvc = GWT.create(ProductsListService.class);
	    }

	    // Set up the callback object.
	    AsyncCallback<ArrayList<String>> callback = new AsyncCallback<ArrayList<String>>() {

			@Override
			public void onFailure(Throwable caught) {
				Window.alert("ERROR WHILE REQUESTING PRODUCT LIST. PLEASE TRY AGAIN LATER");
			}

			@Override
			public void onSuccess(ArrayList<String> result) {
				for (String product:result){
					productChooser.addItem(product);
				}
				productListGrid.setWidget(0, 1, productChooser);
				productChooser.addChangeHandler(new ChangeHandler(){

					@Override
					public void onChange(ChangeEvent event) {

						productUpdateDialogBox.center();
						vPanel.getWidget(1).removeFromParent();
						requestProduct(productChooser.getValue(productChooser.getSelectedIndex()));
						
					}
				});
				refreshProductListDialogBox.hide();
				requestProduct(productChooser.getItemText(productChooser.getSelectedIndex()));

			}
	     
	    };

	    // Make the call to the stock price service.
	    productsListSvc.getProductsList(callback);
		
	}

	private void requestProduct(String productName) {
		// Initialize the service proxy.
	    if (productInfoSvc == null) {
	    	productInfoSvc = GWT.create(ProductInfoService.class);
	    }

	    // Set up the callback object.
	    AsyncCallback<Product> callback = new AsyncCallback<Product>() {

			@Override
			public void onFailure(Throwable caught) {
				Window.alert("FAILED");
			}

			@Override
			public void onSuccess(Product result) {
				vPanel.add(new TaskPanel(result));
				productUpdateDialogBox.hide();
			}
	     
	    };

	    // Make the call to the stock price service.
	    productInfoSvc.getProductInfo(productName, callback);
	}

	private void activeTask(String encryptedId) {
		
		activateTaskDialogBox.setText("Activating task");
		activateTaskDialogBox.setAnimationEnabled(true);
		Label activatingTaskLabel = new Label("Please wait while your task get activated");
		activatingTaskLabel.setStylePrimaryName("dbtext");
		activateTaskDialogBox.add(activatingTaskLabel);
		activateTaskDialogBox.center();
		
		// Initialize the service proxy.
	    if (activateTaskSvc == null) {
	    	activateTaskSvc = GWT.create(ActivateTaskService.class);
	    }

	    // Set up the callback object.
	    AsyncCallback<Boolean> callback = new AsyncCallback<Boolean>() {

			@Override
			public void onFailure(Throwable caught) {
				Window.alert("FAILED TO ACTIVATE");
			}

			@Override
			public void onSuccess(Boolean result) {
				String text = "Activation Confirmed";
				String label = "Your task has been activated.";
				if (!result.booleanValue()){
					text = "Activation Failed";
					label = "Failed due to internal errors, try again later";
				}
				
				activateTaskDialogBox.hide();
				DialogBox confirmMessageDialogBox = new DialogBox();
				confirmMessageDialogBox.setText(text);
				confirmMessageDialogBox.setAnimationEnabled(true);
				Label activateLabel = new Label(label);
				activateLabel.setStylePrimaryName("dbtext");
				
				VerticalPanel activateVPanel = new VerticalPanel();
				activateVPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
				activateVPanel.add(activateLabel);
				
				Button createNewTask = new Button("Create New Task!", new ClickHandler() {
					
					@Override
					public void onClick(ClickEvent event) {
						String href = Location.getHref();
						Location.assign(href.substring(0, href.indexOf("GNC.html?activate")));					
					}
				});
				createNewTask.setStylePrimaryName("subbut");
				activateVPanel.add(createNewTask);
				confirmMessageDialogBox.add(activateVPanel);
				confirmMessageDialogBox.center();
			}
	     
	    };

	    // Make the call to the stock price service.
	    activateTaskSvc.avtivateTask(encryptedId, callback);
	}
}

