package com.ifgi.gnc.client;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;

public class Schedule implements Serializable{

	private Timestamp tInterval;
	private Timestamp tStart;
	private Timestamp tEnd;

	
	public Schedule(){};
	/**
	 * Multiple retrieval events constructor
	 * 
	 * @param tInterval the retrieval interval, example: 1 h -> tInterval.getTime() = 3600000
	 * @param tStart point of first retrieval, example Sat, 09 Jan 2010 11:41:36 GMT -> tInterval.getTime() = 1263037296
	 * @param tEnd point of last retrieval, needs not to match exactly with tStart + k * tInterval
	 */
	public Schedule(Timestamp tInterval, Timestamp tStart, Timestamp tEnd) {
		this.tInterval = tInterval;
		this.tStart = tStart;
		this.tEnd = tEnd;
	}
	
	/**
	 * Single retrieval event constructor
	 * @param when point of the one and only retrieval
	 */
	public Schedule(Timestamp when) {
		this.tStart = when;
		this.tEnd = when;
		this.tInterval = new Timestamp(10000000); // some arbitrary interval
	}
	
	/**
	 * Multiple Retrieval events constructor from explicit events
	 * @param tss a list of equal-interval timestamps with size > 1
	 */
	public Schedule(ArrayList<Timestamp> tss) {
		long start = Long.MAX_VALUE;
		long end = 0;
		for (Timestamp ts : tss) {
			long t = ts.getTime();
			if (t < start) {
				start = t;
			}
			if (t > end) {
				end = t;
			}
		}
		this.tStart = new Timestamp(start);
		this.tEnd = new Timestamp(end);
		if (start == end) {
			this.tInterval = new Timestamp(10000000); // some arbitrary interval
		} else {
			this.tInterval = new Timestamp(Math.abs(tss.get(0).getTime() - tss.get(1).getTime()));	
		}
	}
	
	public Timestamp getInterval() {
		return tInterval;
	}

	public Timestamp getStart() {
		return tStart;
	}

	public Timestamp getEnd() {
		return tEnd;
	}
	
	public ArrayList<Timestamp> getDecomposedSchedule() {
		ArrayList<Timestamp> tss = new ArrayList<Timestamp>();
		Timestamp currentTS = this.tStart;
		while (!tEnd.before(currentTS)) {
			tss.add(currentTS);
			currentTS = new Timestamp(currentTS.getTime() + tInterval.getTime());
		}
		return tss;
	}
}
