package com.ifgi.gnc.client;

import java.sql.Timestamp;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.Window.Location;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.Frame;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.datepicker.client.DatePicker;
import com.ifgi.gnc.server.TaskHandler;

public class TaskPanel extends VerticalPanel {

	private SendTaskServiceAsync sendTaskSvc = GWT
			.create(SendTaskService.class);
	private DialogBox waitMessageDialogBox = new DialogBox();
	private DialogBox errorMessageDialog;
	private Grid dateGrid = new Grid(1, 2);
	private Grid detailInfoGrid = new Grid(1, 3);
	private Grid startTimeGrid = new Grid(1, 5);
	private Grid endTimeGrid = new Grid(1, 5);
	private ListBox interval = new ListBox();
	private ListBox formats = new ListBox();
	private DatePicker startDatePicker = new DatePicker();
	private DatePicker endDatePicker = new DatePicker();
	private TextBox emailTextBox = new TextBox();
	private Timestamp tInterval;
	
	/**
	 * start and end date set?
	 */
	private boolean dateInsert;
	
	/**
	 *start and end date correctness
	 */
	private boolean dateCorrect;
	
//	/**
//	 * start time correctness
//	 */
//	private boolean timeCorrect;
	
	/**
	 * e-mail address set?
	 */
	private boolean mailCorrect;
	
	/**
	 * maximumOfProducts shouldn't be overrun
	 */
	private boolean durationCorrect;
	//TODO How many products are allowed?
	private int maximumOfProducts = 1000;

	public TaskPanel(final Product product) {
		super();
		this.setStylePrimaryName("taskpan");

		Label productNameLabel = new Label(product.getName() + ":");
		productNameLabel.setStylePrimaryName("proname");
		this.add(productNameLabel);

		Label productInfoLabel = new Label(product.getInfo());
		productInfoLabel.setStylePrimaryName("proinfo");
		this.add(productInfoLabel);

		Label specIntervalLabel = new Label("Specify Record Interval:");
		specIntervalLabel.setStylePrimaryName("spechelp");
		this.add(specIntervalLabel);

		// specify dayGrid to the dateGrid
		startTimeGrid.setWidget(0, 0, new HTML("Start at "));
		startTimeGrid.setWidget(0, 2, new HTML(":"));
		startTimeGrid.setWidget(0, 4, new HTML("on"));
		endTimeGrid.setWidget(0, 0, new HTML("Record till "));
		endTimeGrid.setWidget(0, 2, new HTML(":"));
		endTimeGrid.setWidget(0, 4, new HTML("on"));
		ListBox hours = new ListBox();
		ListBox minutes = new ListBox();
		ListBox hours2 = new ListBox();
		ListBox minutes2 = new ListBox();
		int updateIntInMin = product.getUpdateInterval() / 60000;
		if (updateIntInMin <= 60) {
			for (int i = 0; i < 24; i++) {
				Integer hour = new Integer(i);
				hours.addItem(hour.toString());
				hours2.addItem(hour.toString());
			}
			Integer fullHour = 0;
			minutes.addItem(fullHour.toString());
			minutes2.addItem(fullHour.toString());
			for (int i = 0; i < (60 / updateIntInMin) - 1; i++) {
				fullHour = fullHour + updateIntInMin;
				minutes.addItem(fullHour.toString());
				minutes2.addItem(fullHour.toString());
			}
		} else {
			for (int i = 0; i < 1440 / updateIntInMin; i++) {
				Integer hour = new Integer(i * 24 / (1440 / updateIntInMin));
				hours.addItem(hour.toString());
				hours2.addItem(hour.toString());
			}
			minutes.addItem("0");
			minutes2.addItem("0");
		}
		startTimeGrid.setWidget(0, 3, minutes);
		startTimeGrid.setWidget(0, 1, hours);
		endTimeGrid.setWidget(0, 3, minutes2);
		endTimeGrid.setWidget(0, 1, hours2);

		VerticalPanel startVPanel = new VerticalPanel();
		startVPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
		startVPanel.add(startTimeGrid);
		startVPanel.add(startDatePicker);
		dateGrid.setWidget(0, 0, startVPanel);

		VerticalPanel endVPanel = new VerticalPanel();
		endVPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
		endVPanel.add(endTimeGrid);
		endVPanel.add(endDatePicker);
		dateGrid.setWidget(0, 1, endVPanel);

		dateGrid.setCellPadding(5);
		this.add(dateGrid);

		interval.addItem("retrieve all data");
		if (hours.getItemCount() == 24) {
			interval.addItem("hourly");
		}
		interval.addItem("daily");
		interval.addItem("weekly");
		Grid updateGrid = new Grid(1, 2);
		Label specUpdateLabel = new Label("Specify Update Interval:");
		specUpdateLabel.setStylePrimaryName("spechelp");
		updateGrid.setWidget(0, 0, specUpdateLabel);
		updateGrid.setWidget(0, 1, interval);
		this.add(updateGrid);

		double width = ((product.getbBox().getTopRightLon() - product.getbBox()
				.getBottomLeftLon()) * 256)
				/ (product.getbBox().getTopRightLat() - product.getbBox()
						.getBottomLeftLat());
		String widthString = String.valueOf(width + 24);

		Grid mapGrid = new Grid(2, 1);
		Label specAreaLabel = new Label("Specify Area:");
		specAreaLabel.setStylePrimaryName("spechelp");
		Frame frame = new Frame();
		frame.setStyleName("frame");
		frame.setHeight("280px");
		frame.setWidth(widthString + "px");
		frame.setUrl("map.html?left=" + product.getbBox().getBottomLeftLon()
				+ "&bottom=" + product.getbBox().getBottomLeftLat() + "&right="
				+ product.getbBox().getTopRightLon() + "&top="
				+ product.getbBox().getTopRightLat());
		this.add(frame);
		mapGrid.setWidget(0, 0, specAreaLabel);
		mapGrid.setWidget(1, 0, frame);
		this.add(mapGrid);

		for (int i = 0; i < product.getFileFormats().size(); i++) {
			formats.addItem(product.getFileFormats().get(i));
		}
		Grid formatGrid = new Grid(1, 2);
		Label specFormatLabel = new Label("Specify Format:");
		specFormatLabel.setStylePrimaryName("spechelp");
		formatGrid.setWidget(0, 0, specFormatLabel);
		formatGrid.setWidget(0, 1, formats);
		this.add(formatGrid);

		Label specEmailLabel = new Label("E-Mail:");
		specEmailLabel.setStylePrimaryName("spechelp");
		detailInfoGrid.setWidget(0, 0, specEmailLabel);
		detailInfoGrid.setWidget(0, 1, emailTextBox);
		detailInfoGrid.setWidget(0, 2, new HTML("@uni-muenster.de"));
		this.add(detailInfoGrid);

		
		
		// this.add(new HTML(openLayersHTML));
		ClickHandler submitTaskButton = new ClickHandler() {

			
			
			@Override
			public void onClick(ClickEvent event) {
				long startDate = 0;
				long endDate = 0;
				
				// --exception handling--
				try {
					if (interval.getItemText(interval.getSelectedIndex())
							.equals("retrieve all data")) {
						tInterval = new Timestamp(product.getUpdateInterval());
					} else if (interval
							.getItemText(interval.getSelectedIndex()).equals(
									"daily")) {
						tInterval = new Timestamp(86400000);
					} else if (interval
							.getItemText(interval.getSelectedIndex()).equals(
									"weekly")) {
						tInterval = new Timestamp(604800016);
					} else {// hourly
						tInterval = new Timestamp(3600000);
					}
					startDate = startDatePicker.getValue().getTime();
					endDate = endDatePicker.getValue().getTime();
					dateInsert=true;
//					Date date = new Date();
					if (startDate > endDate) {
						dateCorrect = false;
					} else {
						dateCorrect = true;
					}
//					if (startDate - date.getTime() <= 0) {
//						timeCorrect = false;
//					} else {
//						timeCorrect = true;
//					}
					if(((endDate-startDate))/tInterval.getTime()>maximumOfProducts){
						durationCorrect=false;
					}
					else{
						durationCorrect=true;
					}
				} catch (NullPointerException e) {
					dateInsert=false;
				}
				if (emailTextBox.getValue().length() < 1) {
					mailCorrect = false;
				} else {
					mailCorrect = true;
				}
				if (!dateInsert||!dateCorrect || !mailCorrect||!durationCorrect) {
					errorMessageDialog = new DialogBox();
					errorMessageDialog.setWidth("500px");
					errorMessageDialog.setText("An error occured");
					errorMessageDialog.setAnimationEnabled(true);
					String errorMessage = "<b>Please consider the following errors:</b>";
					int counter = 1;
					if(!dateInsert){
						errorMessage = errorMessage +"<br>"+ counter +".Start and End date are not specified.";
						counter++;
					}
					else if(!dateCorrect){
						errorMessage = errorMessage +"<br>"+ counter +".Start and end date have an incorrect order.";
						counter++;
					}
//					else if(!timeCorrect){
//						errorMessage = counter +".Time of start date is earlier than current time.";
//						counter++;
//					}
					if(!durationCorrect&&startDate!=0&&endDate!=0){
						String message = ".The maximum number of products ("+ maximumOfProducts +") is overrun.";
							errorMessage = errorMessage + "<br>"+counter+message;
							counter++;
					}
					if(!mailCorrect){
						String message = ".You did not specify an e-mail address.";
							errorMessage = errorMessage + "<br>"+counter+message;
							counter++;
					}
					HTML errorHTML = new HTML(
							errorMessage);
					errorHTML.setStylePrimaryName("dbtext");
					VerticalPanel errorVPanel = new VerticalPanel();
					errorVPanel
							.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
					errorVPanel.add(errorHTML);
					Button closeButton = new Button("Close");
					errorVPanel.add(closeButton);
					closeButton.addClickHandler(new ClickHandler(){

						@Override
						public void onClick(ClickEvent event) {
							errorMessageDialog.removeFromParent();
						}
					});
					errorMessageDialog.add(errorVPanel);
					errorMessageDialog.center();
				}//end of excpetion handling
				
				
				// everything is correctly filled into the form
				else {
					waitMessageDialogBox = new DialogBox();
					waitMessageDialogBox.setText("Task transmission running");
					waitMessageDialogBox.setAnimationEnabled(true);
					Label activateLabel = new Label(
							"Please wait while your task will be transmitted to the server.");
					activateLabel.setStylePrimaryName("dbtext");
					waitMessageDialogBox.add(activateLabel);
					waitMessageDialogBox.center();
					int startHour = Integer.valueOf(((ListBox) startTimeGrid
							.getWidget(0, 1)).getValue(((ListBox) startTimeGrid
							.getWidget(0, 1)).getSelectedIndex()));
					int startMinute = Integer.valueOf(((ListBox) startTimeGrid
							.getWidget(0, 3)).getValue(((ListBox) startTimeGrid
							.getWidget(0, 3)).getSelectedIndex()));
					int endHour = Integer.valueOf(((ListBox) endTimeGrid
							.getWidget(0, 1)).getValue(((ListBox) endTimeGrid
							.getWidget(0, 1)).getSelectedIndex()));
					int endMinute = Integer.valueOf(((ListBox) endTimeGrid
							.getWidget(0, 3)).getValue(((ListBox) endTimeGrid
							.getWidget(0, 3)).getSelectedIndex()));
					startDate = startDate - 43200000 + startHour * 3600000
							+ startMinute * 60000;// startDate-12Std+Stunden+Minuten
					endDate = endDate - 43200000 + endHour * 3600000
							+ endMinute * 60000;// startDate-12Std+Stunden+Minuten
					Schedule schedule = new Schedule(tInterval, new Timestamp(
							startDate), new Timestamp(endDate));

					// START HERE WITH CREATING NEW TASK
					String id = ((TextBox) (detailInfoGrid.getWidget(0, 1)))
							.getValue()
							+ System.currentTimeMillis();
					id = depointID(id);
					String email = emailTextBox.getValue() + "@uni-muenster.de";
					String format = formats
							.getValue(formats.getSelectedIndex());
					double top = Double.valueOf(getTop());
					double right = Double.valueOf(getRight());
					double bottom = Double.valueOf(getBottom());
					double left = Double.valueOf(getLeft());
					//check if boundingbox values are out of bounds
					if(top>product.getbBox().getTopRightLat()){
						top = product.getbBox().getTopRightLat();
					}
					if(right>product.getbBox().getTopRightLon()){
						right = product.getbBox().getTopRightLon();
					}
					if(bottom<product.getbBox().getBottomLeftLat()){
						bottom = product.getbBox().getBottomLeftLat();
					}
					if(left<product.getbBox().getBottomLeftLon()){
						left = product.getbBox().getBottomLeftLon();
					}
					BBox bBox = new BBox(top, right, bottom, left);
					Task newTask = new Task(email, schedule, product, bBox,
							format, id, true);
					sendTask(newTask);
				}

			}

			private String depointID(String id) {
				id.replace('.', '_');
				return id;
			}
		};
		Button submitButton = new Button("Submit Task", submitTaskButton);
		submitButton.setStylePrimaryName("subbut");
		VerticalPanel submitVPanel = new VerticalPanel();
		submitVPanel
				.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
		submitVPanel.add(submitButton);
		this.add(submitVPanel);

	}

	private static native String getLeft() /*-{
		var left = $wnd.getLeft();

		return left;
	}-*/;

	private static native String getBottom() /*-{
		var bottom = $wnd.getBottom();

		return bottom;
	}-*/;

	private static native String getRight() /*-{
		var right = $wnd.getRight();

		return right;
	}-*/;

	private static native String getTop() /*-{
		var top = $wnd.getTop();

		return top;
	}-*/;

	private void sendTask(final Task newTask) {
		// Initialize the service proxy.
		if (sendTaskSvc == null) {
			sendTaskSvc = GWT.create(SendTaskService.class);
		}

		// Set up the callback object.
		AsyncCallback<Boolean> callback = new AsyncCallback<Boolean>() {

			@Override
			public void onFailure(Throwable caught) {
				Window.alert("FAILED");
			}

			@Override
			public void onSuccess(Boolean result) {
				// Show DialogBox
				if (result.booleanValue()){
					waitMessageDialogBox.removeFromParent();
					DialogBox confirmMessageDialogBox = new DialogBox();
					confirmMessageDialogBox.setText("Confirm Task");
					confirmMessageDialogBox.setAnimationEnabled(true);
					Label activateLabel = new Label("A task activation e-mail was sent to "+newTask.geteMail()+". Please confirm your identity.");
					activateLabel.setStylePrimaryName("dbtext");
					
					VerticalPanel activateVPanel = new VerticalPanel();
					activateVPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
					activateVPanel.add(activateLabel);
					
					HorizontalPanel buttonPanel = new HorizontalPanel();
					Button createNewTask = new Button("Create New Task!", new ClickHandler() {
						
						@Override
						public void onClick(ClickEvent event) {
							Location.assign(Location.getHref());						
						}
					});
					createNewTask.setStylePrimaryName("subbut");
					buttonPanel.add(createNewTask);
					
					Button gotoPerMail = new Button("Go to perMail!", new ClickHandler() {
						
						@Override
						public void onClick(ClickEvent event) {
							Window.open("https://permail.uni-muenster.de/","perMail",null);						
						}
					});
					gotoPerMail.setStylePrimaryName("subbut");
					buttonPanel.add(gotoPerMail);
					
					activateVPanel.add(buttonPanel);
					
					confirmMessageDialogBox.add(activateVPanel);
					confirmMessageDialogBox.center();
				} else {
					waitMessageDialogBox.removeFromParent();
					DialogBox confirmMessageDialogBox = new DialogBox();
					confirmMessageDialogBox.setText("Database Error");
					confirmMessageDialogBox.setAnimationEnabled(true);
					Label activateLabel = new Label("Connection to Database failed! Please try again later.");
					activateLabel.setStylePrimaryName("dbtext");
					
					VerticalPanel activateVPanel = new VerticalPanel();
					activateVPanel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
					activateVPanel.add(activateLabel);
					
					HorizontalPanel buttonPanel = new HorizontalPanel();
					Button createNewTask = new Button("Create New Task!", new ClickHandler() {
						
						@Override
						public void onClick(ClickEvent event) {
							Location.assign(Location.getHref());						
						}
					});
					createNewTask.setStylePrimaryName("subbut");
					buttonPanel.add(createNewTask);
					
					activateVPanel.add(buttonPanel);
					
					confirmMessageDialogBox.add(activateVPanel);
					confirmMessageDialogBox.center();
					
				}
			}

		};
		// Make the call to the stock price service.
		sendTaskSvc.createTask(newTask, callback);
	}

}
