package com.ifgi.gnc.constants;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

public class GODConstants {

	private Logger log=Logger.getLogger(GODConstants.class.getSimpleName());
	private final static String xmlPath = "local\\GODConstants.xml";
	
	// Used by ScriptWriter
	public final String SAF_INPUT_DRIVE = parseForElement("SAF_INPUT_DRIVE");
	public final String SAF_INPUT_DIRECTORY = parseForElement("SAF_INPUT_DIRECTORY");
	public final String MODIS_INPUT_DRIVE = parseForElement("MODIS_INPUT_DRIVE");
	public final String MODIS_INPUT_DIRECTORY = parseForElement("MODIS_INPUT_DIRECTORY");
	public final String MPEF_INPUT_DRIVE = parseForElement("MPEF_INPUT_DRIVE");
	public final String MPEF_INPUT_DIRECTORY = parseForElement("MPEF_INPUT_DIRECTORY");
	public final String GDAL_DIRECTORY = parseForElement("GDAL_DIRECTORY");
	public final String ILWIS_DIRECTORY = parseForElement("ILWIS_DIRECTORY");
	public final String UTIL_DIRECTORY = parseForElement("UTIL_DIRECTORY");

	// Used by IlwisUtil
	public final String BATCH_FILEPATH = parseForElement("BATCH_FILEPATH");
	public final String SCRIPT_DIR = parseForElement("SCRIPT_DIR");
	public final String WORKING_DIR = parseForElement("WORKING_DIR");
	public final String XY_FILEPATH = parseForElement("XY_FILEPATH");

	// Used by TaskHandler

	// internal image repository
	public final String USER_DIR = parseForElement("USER_DIR");

	// external image repository
	public final String USER_URL = parseForElement("USER_URL");

	// Database information
	public final String DB_DRIVER = parseForElement("DB_DRIVER");
	public final String DB_HOST = parseForElement("DB_HOST");
	public final String DB_PORT = parseForElement("DB_PORT");
	public final String DB = parseForElement("DB");
	public final String DB_USER = parseForElement("DB_USER");
	public final String DB_PW = parseForElement("DB_PW");

	// SQL-relevant information
	public final String DB_TABLE = parseForElement("DB_TABLE");
	public final String DB_COL_EMAIL = parseForElement("DB_COL_EMAIL");
	public final String DB_COL_EXECUTION = parseForElement("DB_COL_EXECUTION");
	public final String DB_COL_PRODUCT = parseForElement("DB_COL_PRODUCT");
	public final String DB_COL_BOTTOMLEFTLON = parseForElement("DB_COL_BOTTOMLEFTLON");
	public final String DB_COL_BOTTOMLEFTLAT = parseForElement("DB_COL_BOTTOMLEFTLAT");
	public final String DB_COL_TOPRIGHTLON = parseForElement("DB_COL_TOPRIGHTLON");
	public final String DB_COL_TOPRIGHTLAT = parseForElement("DB_COL_TOPRIGHTLAT");
	public final String DB_COL_FORMAT = parseForElement("DB_COL_FORMAT");
	public final String DB_COL_TASKID = parseForElement("DB_COL_TASKID");
	public final String DB_COL_SCRIPT = parseForElement("DB_COL_SCRIPT");
	
	private String parseForElement(String elementName) {
		String constant = "";
		try {
			String path = xmlPath;
			if (!new File(path).exists()) {
				path = "..\\" + path;
			}
			File file = new File(path);
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.isIgnoringElementContentWhitespace();
			DocumentBuilder db = dbf.newDocumentBuilder();
			org.w3c.dom.Document doc = db.parse(file);
			doc.normalizeDocument();
			// get Node value of elementName
			constant = doc.getElementsByTagName(elementName).item(0)
					.getFirstChild().getNodeValue();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			log.info(e.toString());
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			log.info(e.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			log.info(e.toString());
		}
		constant = cleanPath(constant); 
		return constant;
	}

	// remove \n or \t
	private static String cleanPath(String constant) {
		String cleaned = "";
		StringBuffer sb = new StringBuffer(constant);
		for (int i = 0; i < sb.length(); i++) { 
			String str = sb.substring(i, i+1); 
			if (!(str.equals("\n") || str.equals("\t"))) {
				cleaned += str;
			}
		}
		return cleaned;
	}
}
