package com.ifgi.gnc.constants;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import com.ifgi.gnc.client.BBox;
import com.ifgi.gnc.client.Product;

public class Products {

	private Logger log=Logger.getLogger(GODConstants.class.getSimpleName());
	private final static String xmlPath = "local\\Products.xml";
	
	public final ArrayList<Product> products = getProducts();

	public Product byName(String name) {
		for (Product p : products) {
			if (p.getName().equals(name)) {
				return p;
			}
		}
		return null;
	}
	
	private ArrayList<Product> getProducts() {
		ArrayList<Product> products = new ArrayList<Product>();
		for (String p : parseXML()) {
			products.add(parseForProduct(p));
		}
		return products;
	}
	
	private ArrayList<String> parseXML() {
		ArrayList<String> result = new ArrayList<String>();
		try {
			String path = xmlPath;
			if (!new File(path).exists()) {
				path = "..\\" + path;
			}
			File file = new File(path);
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document doc = db.parse(file);
			// parse the XML document into a DOM
			for (int i = 0; i < doc.getElementsByTagName("name").getLength(); i++) {
				String nameValue = doc.getElementsByTagName("name").item(i).getFirstChild().getNodeValue();
				result.add(nameValue);
			}
		} catch (ParserConfigurationException e) {
			log.info(e.toString());
		} catch (SAXException e) {
			log.info(e.toString());
		} catch (IOException e) {
			log.info(e.toString());
		}
		return result;
	}

	private Product parseForProduct(String name) {
		Product product = new Product();
		try {
			String path = xmlPath;
			if (!new File(path).exists()) {
				path = "..\\" + path;
			}
			File file = new File(path);
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			org.w3c.dom.Document doc = db.parse(file);

			for (int i = 0; i < doc.getElementsByTagName("name")
			.getLength(); i++) {
				if (doc.getElementsByTagName("name").item(i)
						.getFirstChild().getNodeValue().equals(name)) {
					// get name
					String nameValue = doc.getElementsByTagName("name")
					.item(i).getFirstChild().getNodeValue();
					product.setName(nameValue);
					// get update interval
					int updateIntervalValue = Integer.valueOf(doc.getElementsByTagName("updateInterval").item(i).getFirstChild().getNodeValue());
					product.setUpdateInterval(updateIntervalValue);
					// get BBox values
					// get latitude value of top left coordinate of BBox
					double topLat = Double.valueOf(doc
							.getElementsByTagName("topRightLat").item(i)
							.getFirstChild().getNodeValue());
					// get longitude value of top left coordinate of BBox
					double topLong = Double.valueOf(doc
							.getElementsByTagName("topRightLon").item(i)
							.getFirstChild().getNodeValue());
					// get latitude value of low right coordinate of BBox
					double lowLat = Double.valueOf(doc
							.getElementsByTagName("bottomLeftLat").item(i)
							.getFirstChild().getNodeValue());
					// get longitude value of low right coordinate of BBox
					double lowLong = Double.valueOf(doc
							.getElementsByTagName("bottomLeftLon").item(i)
							.getFirstChild().getNodeValue());
					product.setbBox(new BBox(topLat,topLong,lowLat,lowLong));
					// get supported file formats
					Element formatsNode = (Element)(doc.getElementsByTagName(
					"fileFormats").item(i));
					ArrayList<String> fileFormats = new ArrayList<String>();
					//parse each childnode for "fileformats"
					for (int k = 0; k < formatsNode.getElementsByTagName("format").getLength(); k++) {
						if (formatsNode.getElementsByTagName("format").item(k).getFirstChild().getNodeValue().toLowerCase().compareTo("tif")==0) {
							fileFormats.add("tif");
							// TODO add other formats as conditions
						}
					}
					product.setFileFormats(fileFormats);
					// get info for the product
					String infoValue = doc.getElementsByTagName("info")
					.item(i).getFirstChild().getNodeValue();
					product.setInfo(infoValue);
				}
			}

		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return product;
	}
}
