package com.ifgi.gnc.server;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;

import com.ifgi.gnc.client.BBox;
import com.ifgi.gnc.constants.GODConstants;

public class GODUtil {
	private static GODConstants constant = new GODConstants();
	
	public static BBoxXY convertBBoxLatLonToMapXY(BBox bbox, String georefName) throws IOException, InterruptedException{
		double botleftX = 0.0, botleftY = 0.0, toprightX = 0.0, toprightY = 0.0;
		
		double tempLat = bbox.getBottomLeftLat(), tempLon = bbox.getBottomLeftLon();
		System.out.println("Convert BotLeft XY");
		generateXY(georefName, tempLat, tempLon);
		System.out.println("Convert BotLeft XY done");
		
		BufferedReader br = new BufferedReader(new FileReader(constant.XY_FILEPATH));
		String[] coordStr = br.readLine().split(",");
		botleftX = Double.parseDouble(coordStr[0]);
		botleftY = Double.parseDouble(coordStr[1]);
		br.close();
		
		tempLat = bbox.getTopRightLat(); tempLon = bbox.getTopRightLon();
		System.out.println("Convert TopRight XY");
		generateXY(georefName, tempLat, tempLon);
		System.out.println("Convert TopRight XY done");
		
		br = new BufferedReader(new FileReader(constant.XY_FILEPATH));
		coordStr = br.readLine().split(",");
		toprightX = Double.parseDouble(coordStr[0]);
		toprightY = Double.parseDouble(coordStr[1]);
		br.close();
		
		return new BBoxXY(toprightX, toprightY, botleftX, botleftY);
	}

	private static void generateXY(String georefName, double tempLat, double tempLon) throws IOException, InterruptedException {
		File batFile = new File(constant.BATCH_FILEPATH);
		if(batFile.exists())
			batFile.delete();
		batFile.createNewFile();
		
		PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(batFile)));
		pw.println("@echo off");
		pw.println("set ilwispath="+constant.ILWIS_DIRECTORY);
		pw.println("set scriptdir="+constant.SCRIPT_DIR);
		pw.println("set wkdir="+constant.WORKING_DIR);
		pw.println("set lon="+tempLon);
		pw.println("set lat="+tempLat);
		pw.println("set newcoord="+georefName+".csy");
		pw.println();
		pw.println();		
		pw.println(":STARTCONVERT");
		pw.println("del /F %wkdir%\\var_tabXY.txt");
		pw.println("START /REALTIME /WAIT %ilwispath%\\Ilwis.exe -C run '%scriptdir%\\TransformLATLON'.isl %wkdir% %lon% %lat% %newcoord%");
//		pw.println("REM Wait 5 seconds to be finished");
//		pw.println("ping -n 5 127.0.0.1");
		pw.println("IF NOT EXIST \"%wkdir%\\var_tabXY.txt\" (");
//		pw.println("	echo fail");
		pw.println("	TASKKILL /f /im Ilwis.exe");
//		pw.println("	REM Repeat if Ilwis failed");
		pw.println("	GOTO STARTCONVERT");
		pw.println(")");
		pw.println();
		pw.close();
		
		Runtime runtime = Runtime.getRuntime();
		Process pr = runtime.exec(constant.BATCH_FILEPATH);
		// Alternative waiting method, replacement for pr.waitFor()
		BufferedReader br = new BufferedReader(new InputStreamReader(pr.getInputStream()));
		String cmdLine = br.readLine();
		while(cmdLine!=null){
			System.out.println(cmdLine);
			cmdLine = br.readLine();
		}
		br.close();
	}

	public static String getLongFileName(Timestamp ts){
		SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmm");
		
		return formatter.format(ts);
	}
	
	public static String getDayNumber(Timestamp ts){
		SimpleDateFormat formatter = new SimpleDateFormat("DDD");
		
		return formatter.format(ts);
	}
}
