package com.ifgi.gnc.server;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.ifgi.gnc.client.BBox;
import com.ifgi.gnc.client.Product;

public class ProductInstance {

	private final Product metaData;
	private final BBox bbox;
	private final Timestamp ts;
	private final String format;
	
	public ProductInstance(Product metaData, BBox bbox, Timestamp ts, String format) {
		this.metaData = metaData;
		this.bbox = bbox;
		this.ts = ts;
		this.format = format;
	}
	
	public Product getMetaData() {
		return metaData;
	}

	public BBox getBBox() {
		return bbox;
	}

	public Timestamp getTs() {
		return ts;
	}

	public String getFormat() {
		return format;
	}
	
	public boolean write2File(String fullpath) {
		Document doc = null;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		try {
			builder = factory.newDocumentBuilder();
			doc = builder.newDocument();
			String pins = "http://gnc-vs01.uni-muenster.de/god/ProductInstance";
			String gmlns = "http://www.opengis.net/gml/3.2";
			String gmlLoc = "http://schemas.opengis.net/gml/3.2.1/gml.xsd";
			Element piE = doc.createElementNS(pins, "ProductInstance");
			doc.appendChild(piE);
			piE.setAttribute("xmlns:gml", gmlns);
			piE.setAttributeNS("http://www.w3.org/2001/XMLSchema-instance", "xsi:schemaLocation",
								pins + " " + pins + ".xsd " + gmlns + " " + gmlLoc);
			// Create product tag
			Element productE = doc.createElement("Product");
			Node productN = doc.createTextNode(this.metaData.getName());
			productE.appendChild(productN);
			piE.appendChild(productE);
			// Create timestamp tag
			Element tsE = doc.createElement("Timestamp");
			Date d = new Date(this.ts.getTime());
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss");
			Node tsN = doc.createTextNode(sdf.format(d));
			tsE.appendChild(tsN);
			piE.appendChild(tsE);
			// Create format tag
			Element formatE = doc.createElement("FileFormat");
			Node formatN = doc.createTextNode(this.format);
			formatE.appendChild(formatN);
			piE.appendChild(formatE);
			// Create bbox tag
			String upper = this.bbox.getTopRightLat() + " " + this.bbox.getTopRightLon();
			String lower = this.bbox.getBottomLeftLat() + " " + this.bbox.getBottomLeftLon();
			Element bboxE = doc.createElement("BBox");
			Element upperE = doc.createElement("gml:upperCorner");
			Element lowerE = doc.createElement("gml:lowerCorner");
			Node upperN = doc.createTextNode(upper);
			Node lowerN = doc.createTextNode(lower);
			upperE.appendChild(upperN);
			lowerE.appendChild(lowerN);
			bboxE.appendChild(lowerE);
			bboxE.appendChild(upperE);
			piE.appendChild(bboxE);
			doc.normalizeDocument();
            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            DOMSource source = new DOMSource(doc);
            FileOutputStream os = new FileOutputStream(new File(fullpath));
            StreamResult result = new StreamResult(os);
            transformer.transform(source, result);
		} catch (ParserConfigurationException e) {
			return false;
		} catch (TransformerConfigurationException e) {
			return false;
		} catch (TransformerFactoryConfigurationError e) {
			return false;
		} catch (FileNotFoundException e) {
			return false;
		} catch (TransformerException e) {
			return false;
		}
		return true;
	}

	public boolean isConsistent() {
		return checkBBox() && checkTimestamp() && checkFormat();
	}

	private boolean checkTimestamp() {
		// TODO Auto-generated method stub
		return false;
	}

	private boolean checkFormat() {
		// TODO Auto-generated method stub
		return false;
	}

	private boolean checkBBox() {
		// TODO Auto-generated method stub
		return false;
	}

}
