package com.ifgi.gnc.server;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.logging.Logger;
import com.ifgi.gnc.client.BBox;
import com.ifgi.gnc.client.Product;
import com.ifgi.gnc.client.Schedule;
import com.ifgi.gnc.client.Task;
import com.ifgi.gnc.constants.GODConstants;
import com.ifgi.gnc.constants.Products;

public class TaskHandler {

	private static Logger logger = Logger.getLogger("TaskHandler_log");
	private GODConstants constants = new GODConstants();
	private Products products = new Products();
	
	public boolean registerTask(Task t) {
		try {
			writeDBJobs(t);
		} catch (SQLException e) {
			logger.severe("Unable to create database connection: SQL error code " + e.getErrorCode());
			return false;
		}
		return true;
	}

	public boolean activateTask(String id) {
		Task t;
		String scriptFile = "unknown";
		try {
			t = readDBTask(id);
			ArrayList<ProductInstance> pis = task2PIs(t);
			int jobCount = 0;
			for (ProductInstance pi : pis) {
				jobCount++;
				scriptFile = id + "_" + jobCount;
				writeScriptPointer2DB(scriptFile, id, pi.getTs());
			}
			logger.info(pis.size() + " new jobs activated for task " + id + ".");
		} catch (SQLException e) {
			logger.severe("Unable to create database connection: SQL error code " + e.getErrorCode());
			return false;
		}
		return true;
		
	}
	
	

	private Connection getDBConnection() throws SQLException {
		try {
			Class.forName(constants.DB_DRIVER);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		Connection conn = null;
		String url = "jdbc:postgresql:" + (constants.DB_HOST != null ? ("//" + constants.DB_HOST) + (constants.DB_PORT != null ? ":" + constants.DB_PORT : "") + "/" : "") + constants.DB;
		try {
			conn = DriverManager.getConnection(url, constants.DB_USER, constants.DB_PW);
		} catch (SQLException e) {
			throw e;
		}
		return conn;
	}

	private void writeDBJobs(Task t) throws SQLException {
		Connection conn = getDBConnection();
		Statement stm = null;
		ArrayList<Timestamp> jobs = t.getSchedule().getDecomposedSchedule();
		try {
			stm = conn.createStatement();
			String stub = 	"INSERT INTO " +
							constants.DB_TABLE + " (" +
							constants.DB_COL_EMAIL + ", " + 
							constants.DB_COL_EXECUTION + ", " +
							constants.DB_COL_FORMAT + ", " +
							constants.DB_COL_BOTTOMLEFTLON + ", " +
							constants.DB_COL_BOTTOMLEFTLAT + ", " +
							constants.DB_COL_TOPRIGHTLON + ", " +
							constants.DB_COL_TOPRIGHTLAT + ", " +
							constants.DB_COL_PRODUCT + ", " +
							constants.DB_COL_SCRIPT + ", " +
							constants.DB_COL_TASKID + ")" +
							" VALUES (";
			for (Timestamp ts : jobs) {
				String job = stub;
				job += "'" + t.geteMail() + "', ";
				job += "TIMESTAMP '" + ts + "', ";
				job += "'" + t.getFormat() + "', ";
				job += t.getBbox().getBottomLeftLon() + ", ";
				job += t.getBbox().getBottomLeftLat() + ", ";
				job += t.getBbox().getTopRightLon() + ", ";
				job += t.getBbox().getTopRightLat() + ", ";
				job += "'" + t.getProduct().getName() + "', ";
				job += "NULL, ";
				job += "'" + t.getId() + "')";
				stm.executeUpdate(job);
			}
		} catch (SQLException e) {
			logger.warning("Unable to execute update: SQL error code " + e.getErrorCode());
		}
		try {
			conn.close();
		} catch (SQLException e) {
			logger.warning("Unable to close database connection.");
		}
		logger.info("Task " + t.getId() + " stored in database as " + jobs.size() + " individual jobs.");
	}

	private void writeScriptPointer2DB(String scriptFile, String id, Timestamp ts) throws SQLException {
		Connection conn = getDBConnection();
		Statement stm = null;
		try {
			stm = conn.createStatement();
			stm.executeUpdate(	"UPDATE " + constants.DB_TABLE +
								" SET " + constants.DB_COL_SCRIPT + " = '" + scriptFile +
								"' WHERE " + constants.DB_COL_TASKID + " = '" + id +
								"' AND " + constants.DB_COL_EXECUTION + " = '" + ts + "'");
		} catch (SQLException e) {
			logger.warning("Unable to execute update: SQL error code " + e.getErrorCode());
		}
	}

	private Task readDBTask(String id) throws SQLException {
		Task t = null;
		Connection conn = getDBConnection();
		Statement stm = null;
		try {
			stm = conn.createStatement();
			ResultSet rs = stm.executeQuery("SELECT * FROM " + constants.DB_TABLE + " WHERE " + constants.DB_COL_TASKID + " = '" + id + "'");
			ArrayList<Timestamp> tss = new ArrayList<Timestamp>();
			boolean first = true;
			String eMail = null;
			Product product = null;
			double topRightLat = 0;
			double topRightLon = 0;
			double bottomLeftLat = 0;
			double bottomLeftLon = 0;
			String format = null;
			while (rs.next()) {
				tss.add(rs.getTimestamp(constants.DB_COL_EXECUTION));
				if (first) {
					// some arbitrary job, since it reflects the task except from the timestamps
					eMail = rs.getString(constants.DB_COL_EMAIL);
					product = products.byName(rs.getString(constants.DB_COL_PRODUCT));
					topRightLat = rs.getDouble(constants.DB_COL_TOPRIGHTLAT);
					topRightLon = rs.getDouble(constants.DB_COL_TOPRIGHTLON);
					bottomLeftLat = rs.getDouble(constants.DB_COL_BOTTOMLEFTLAT);
					bottomLeftLon = rs.getDouble(constants.DB_COL_BOTTOMLEFTLON);
					format = rs.getString(constants.DB_COL_FORMAT);
					first = false;
				}
			}
			BBox bbox = new BBox(topRightLat, topRightLon, bottomLeftLat, bottomLeftLon);
			Schedule schedule = new Schedule(tss);
			t = new Task(eMail, schedule, product, bbox, format, id, false);
		} catch (SQLException e) {
			logger.warning("Unable to execute query: SQL error code " + e.getErrorCode());
		}
		try {
			conn.close();
		} catch (SQLException e) {
			logger.warning("Unable to close database connection.");
		}
		return t;
	}

	private ArrayList<ProductInstance> task2PIs(Task t) {
		Product metaData = t.getProduct();
		BBox bbox = t.getBbox();
		String format = t.getFormat();
		ArrayList<Timestamp> tss = t.getSchedule().getDecomposedSchedule();
		ArrayList<ProductInstance> result = new ArrayList<ProductInstance>();
		for (Timestamp ts : tss) {
			ProductInstance pi = new ProductInstance(metaData, bbox, ts, format);
			result.add(pi);
		}
		return result;
	}
}
