package com.ifgi.gnc.server.writer;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import com.ifgi.gnc.client.BBox;
import com.ifgi.gnc.client.Product;
import com.ifgi.gnc.server.BBoxXY;
import com.ifgi.gnc.server.ProductInstance;
import com.ifgi.gnc.constants.GODConstants;

public abstract class ScriptWriter {
		
	protected GODConstants constants = new GODConstants();
	protected File scriptFile;
	protected File outputDir;
	
	protected abstract void writeScript(ProductInstance pi) throws IOException, InterruptedException;

	// scriptFile have .bat
	public void createScript(ProductInstance pi, String scriptFileStr, String scriptDirStr, String outputDirStr) throws IOException, InterruptedException{
		System.out.println("createScript");
		String scriptFullPath = scriptDirStr + File.separatorChar + scriptFileStr;
		this.scriptFile = new File(scriptFullPath);
		this.outputDir = new File(outputDirStr);
		if (this.scriptFile.exists()) {
			this.scriptFile.delete();
		}
		this.scriptFile.createNewFile();
		writeScript(pi);
	}

	protected void writeGeneralTemplate(String filename, 
			String outputFilename,
			BBoxXY bBoxXY,
			String outDriveStr,
			String outDirStr,
			String inputDrive,
			String inputDirectory,
			String scriptName,
			String finalMapName,
			String productName,
			String[] tempFiles,
			String checkFilename) throws IOException {
		PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(scriptFile)));
		String windowTitle = "C:\\Windows\\system32\\cmd.exe - "+constants.SCRIPT_DIR+File.separator+scriptName+".bat";
		String exportScriptName = "MapSubMapCoords";
		
		// CREATE MAP
		pw.println();
		pw.println("@echo off");
		pw.println("set scriptDir="+constants.SCRIPT_DIR);
		pw.println("set longfilename="+filename);
		pw.println("set shortfilename1=%longfilename:~0,12%");
		pw.println("set InputDrive="+inputDrive);
		pw.println("set InputDir="+inputDirectory);
		pw.println("set OutputDrive="+outDriveStr);
		pw.println("set OutputDir="+outDirStr);
		pw.println("set gdalDir="+constants.GDAL_DIRECTORY);
		pw.println("set IlwDir="+constants.ILWIS_DIRECTORY);
		pw.println("set UtilDir="+constants.UTIL_DIRECTORY);
		pw.println();
		pw.println("set /A Counter=1");
		pw.println(":STARTIMPORT");
		pw.println("IF NOT EXIST "+checkFilename+" EXIT 1");
		pw.println("START /WAIT %scriptDir%\\"+scriptName+".bat %longfilename% %InputDrive% %InputDir% %OutputDrive% %OutputDir% %gdalDir% %IlwDir% %UtilDir%");
		// EXPORT TIF
		pw.println("set SubMapInputDir="+outDriveStr+File.separator+outDirStr);
		pw.println("set SubMapSrcMap="+finalMapName+filename);//final map name, if script is finished
		pw.println("set SubMapDstMap="+outputFilename);
		pw.println("set bottomleft_x="+bBoxXY.getBottomLeftX());
		pw.println("set bottomleft_y="+bBoxXY.getBottomLeftY());
		pw.println("set topright_x="+bBoxXY.getTopRightX());
		pw.println("set topright_y="+bBoxXY.getTopRightY());
		pw.println();
		pw.println("set /A Counter2=1");
		pw.println(":STARTEXPORT");
		pw.println("START /REALTIME /WAIT %IlwDir%\\Ilwis.exe -C %SubMapInputDir%\\stretch%SubMapSrcMap%.mpr{dom=image.dom;vr=0:255} := MapStretchLinear(%SubMapInputDir%\\%SubMapSrcMap%,1.000000,image.dom)");
		pw.println("START /REALTIME /WAIT %IlwDir%\\Ilwis.exe -C %SubMapInputDir%\\%SubMapDstMap%.mpr:=MapSubMapCoords('%SubMapInputDir%\\stretch%SubMapSrcMap%'.mpr,%bottomleft_x%,%bottomleft_y%,%topright_x%,%topright_y%)");
		pw.println("START /REALTIME /WAIT %IlwDir%\\Ilwis.exe -C export TIFF(%SubMapInputDir%\\%SubMapDstMap%.mpr,%SubMapInputDir%\\%SubMapDstMap%)");
		pw.println(":END");
		writeDeleteTempFile(pw, tempFiles);
		pw.close();
	}
	
	protected void writeGeneralTemplateDayNumber(String filename, 
			String outputFilename,
			BBoxXY bBoxXY,
			String outDriveStr,
			String outDirStr,
			String inputDrive,
			String inputDirectory,
			String scriptName,
			String finalMapName,
			String dayNumber,
			String productName,
			String[] tempFiles,
			String checkFilename) throws IOException {
		PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(scriptFile)));
		String windowTitle = "C:\\Windows\\system32\\cmd.exe - "+constants.SCRIPT_DIR+File.separator+scriptName+".bat";
		// CREATE MAP
		pw.println("echo executing " + scriptName + " ...");
		pw.println("@echo off");
		pw.println("set scriptDir="+constants.SCRIPT_DIR);
		pw.println("set longfilename="+filename);
		pw.println("set shortfilename1=%longfilename:~0,12%");
		pw.println("set InputDrive="+inputDrive);
		pw.println("set InputDir="+inputDirectory);
		pw.println("set OutputDrive="+outDriveStr);
		pw.println("set OutputDir="+outDirStr);
		pw.println("set gdalDir="+constants.GDAL_DIRECTORY);
		pw.println("set IlwDir="+constants.ILWIS_DIRECTORY);
		pw.println("set UtilDir="+constants.UTIL_DIRECTORY);
		pw.println("set DayNumber="+dayNumber);
		pw.println();
		pw.println("set /A Counter=1");
		pw.println(":STARTIMPORT");
		pw.println("IF NOT EXIST "+checkFilename+" EXIT 1");
		pw.println("START /WAIT %scriptDir%\\"+scriptName+".bat %longfilename% %InputDrive% %InputDir% %OutputDrive% %OutputDir% %gdalDir% %IlwDir% %UtilDir% %DayNumber%");
		pw.println("");
//		// EXPORT TIF
		pw.println("set SubMapInputDir="+outDriveStr+File.separator+outDirStr);
		pw.println("set SubMapSrcMap="+finalMapName+filename);//final map name, if script is finished
		pw.println("set SubMapDstMap="+outputFilename);
		pw.println("set bottomleft_x="+bBoxXY.getBottomLeftX());
		pw.println("set bottomleft_y="+bBoxXY.getBottomLeftY());
		pw.println("set topright_x="+bBoxXY.getTopRightX());
		pw.println("set topright_y="+bBoxXY.getTopRightY());
		pw.println();
		pw.println("set /A Counter2=1");
		pw.println(":STARTEXPORT");
		pw.println("START /REALTIME /WAIT %IlwDir%\\Ilwis.exe -C %SubMapInputDir%\\stretch%SubMapSrcMap%.mpr{dom=image.dom;vr=0:255} := MapStretchLinear(%SubMapInputDir%\\%SubMapSrcMap%,1.000000,image.dom)");
		pw.println("START /REALTIME /WAIT %IlwDir%\\Ilwis.exe -C %SubMapInputDir%\\%SubMapDstMap%.mpr:=MapSubMapCoords('%SubMapInputDir%\\stretch%SubMapSrcMap%'.mpr,%bottomleft_x%,%bottomleft_y%,%topright_x%,%topright_y%)");
		pw.println("START /REALTIME /WAIT %IlwDir%\\Ilwis.exe -C export TIFF(%SubMapInputDir%\\%SubMapDstMap%.mpr,%SubMapInputDir%\\%SubMapDstMap%)");
		pw.println(":END");
		writeDeleteTempFile(pw, tempFiles);
		pw.println("EXIT 1");
		pw.println();
		pw.close();
	}

	private void writeDeleteTempFile(PrintWriter pw, String[] tempFile) {
		for(int i=0;i < tempFile.length;i++)
			pw.println("del /f /q %OutputDrive%\\%OutputDir%\\"+tempFile[i]);
		pw.println("del /f /q %SubMapInputDir%\\%SubMapSrcMap%.*");
		pw.println("del /f /q %SubMapInputDir%\\stretch%SubMapSrcMap%.mp*");
		pw.println("del /f /q %SubMapInputDir%\\%SubMapDstMap%.mp*");
		pw.println("del /f /q %SubMapInputDir%\\%SubMapDstMap%.grf");
		pw.println("del /f /q %SubMapInputDir%\\ilwis.log");
		pw.println("del /f /q *.grf");
		pw.println();
	}
	
	public static ScriptWriter getScriptWriter(String productId){
		if(productId.equals("Africa_SAF_Albedo"))
			return new AlbedoScriptWriter();
		else if (productId.equals("Africa_SAF_DSSF"))
			return new DSSFScriptWriter();
		else if (productId.equals("Africa_SAF_DSLF"))
			return new DSLFScriptWriter();
		else if (productId.equals("Africa_SAF_FVC"))
			return new FVCScriptWriter();
		else if (productId.equals("Africa_SAF_LAI"))
			return new LAIScriptWriter();
		else if (productId.equals("Africa_SAF_LST"))
			return new LSTScriptWriter();
		else if (productId.equals("Africa_SAF_fAPAR"))
			return new fAPARScriptWriter();
		else if (productId.equals("MODIS_Fire"))
			return new MODISScriptWriter();
		else if (productId.equals("MPEF_FIRA"))
			return new FIRAScriptWriter();
		
		return null;
	}
}
